package com.qmth.distributed.print.api;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.excel.ExcelField;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.distributed.print.business.service.TCScoreNormalService;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.log.CustomizedOperationTypeEnum;
import com.qmth.teachcloud.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 平常作业成绩表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Api(tags = "课程目标达成度-成绩管理-平常作业成绩Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_COURSE_DEGREE)
public class TCScoreNormalController {

    @Resource
    TCScoreNormalService tcScoreNormalService;

    @ApiOperation(value = "成绩管理列表")
    @RequestMapping(value = "/score/list", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = Object.class)})
    public Object scoreList(@ApiParam(value = "学期ID", required = true) @RequestParam Long semesterId,
                            @ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                            @ApiParam(value = "课程编码") @RequestParam(required = false) String courseCode,
                            @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                            @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(tcScoreNormalService.scoreList(new Page<>(pageNumber, pageSize), semesterId, examId, courseCode));
    }

    @ApiOperation(value = "导入平时成绩-模板下载")
    @RequestMapping(value = "/score/template_download", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.EXPORT)
    @ApiResponses({@ApiResponse(code = 200, message = "下载成功", response = Object.class)})
    public void templateDownload(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                                 @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                                 @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                                 @ApiParam(value = "试卷类型", required = true) @RequestParam String paperType) throws IOException {
        List<ExcelField> excelFieldList = new ArrayList<>();
        excelFieldList.add(new ExcelField("examNumber", "学号", true));
        excelFieldList.add(new ExcelField("name", "姓名", true));
        excelFieldList.add(new ExcelField("score1", "作业1", true));
        excelFieldList.add(new ExcelField("score2", "作业2", true));
        excelFieldList.add(new ExcelField("score3", "作业3", false));
        tcScoreNormalService.downLoadExaminationTemplate("1、所有字段均为必填字段；\n" +
                "2、平时成绩各项数据均需要填写；\n" +
                "3、请不要删除此行，也不要删除模板中的任何列。\n" +
                "4、使用前请先删除样例数据。", "sheet1", "平时成绩导入模版", excelFieldList);
    }

    @ApiOperation(value = "导入平时成绩")
    @RequestMapping(value = "/score/import", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.IMPORT)
    @ApiResponses({@ApiResponse(code = 200, message = "导入成功", response = EditResult.class)})
    public Object scoreImport(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "平时成绩列表")
    @RequestMapping(value = "/score/normal/list", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    @ApiResponses({@ApiResponse(code = 200, message = "导入成功", response = EditResult.class)})
    public Object scoreNormalList(@ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                  @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "平时成绩编辑")
    @RequestMapping(value = "/score/normal/edit", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.EDIT)
    @ApiResponses({@ApiResponse(code = 200, message = "编辑", response = EditResult.class)})
    public Object scoreNormalEdit(@ApiParam(value = "主键", required = true) @RequestParam Long id) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "平时成绩保存")
    @RequestMapping(value = "/score/normal/save", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "保存", response = EditResult.class)})
    public Object scoreNormalSave(@ApiParam(value = "主键", required = true) @RequestParam Long id,
                                  @ApiParam(value = "平时成绩json", required = true) @RequestParam String socreNormal) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "平时成绩启用/禁用")
    @RequestMapping(value = "/score/normal/enable", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "编辑", response = EditResult.class)})
    public Object scoreNormalEnable(@ApiParam(value = "主键", required = true) @RequestParam Long id,
                                    @ApiParam(value = "启用/禁用", required = true) @RequestParam Boolean enable) throws IOException {
        return ResultUtil.ok(true);
    }
}
